--
-- WeidemannHoftrac916DM
-- Specialization for Weidemann Hoftrac 916 D/M
--
-- @author    PES 4ever
-- @date      11/07/11
-- @version   1.0
--
-- For support and documentation visit www.modding-society.de
-- Copyright (C) PES 4ever, All Rights Reserved.
--

WeidemannHoftrac916DM = {};

Vehicle.registerJointType("weidemannCProfile");

function WeidemannHoftrac916DM.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Cylindered, specializations);
end;

function WeidemannHoftrac916DM:load(xmlFile)

	self.setIsAutoSteerBackStopped = SpecializationUtil.callSpecializationsFunction("setIsAutoSteerBackStopped");

	-- Steering auto rotate back
	self.autoSteerBack = {};
	self.autoSteerBack.isStopped = false;
	self.autoSteerBack.maxRotatedTimeSpeedSave = self.maxRotatedTimeSpeed;
	
	-- Steering last rotated time
	self.rotatedTimeLast = self.rotatedTime;

	-- Tilt cylinder
	self.tiltCyl = {};
	self.tiltCyl.transMinSave = self.movingTools[2].transMin;
	self.tiltCyl.transMaxSave = self.movingTools[2].transMax;
	
	-- Motor rpm
	self.motorSoundPitchOffsetSave = self.motorSoundPitchOffset;
	
end;

function WeidemannHoftrac916DM:delete()
end;

function WeidemannHoftrac916DM:readStream(streamId, connection)
    local isStopped = streamReadBool(streamId);
    self:setIsAutoSteerBackStopped(isStopped, true);
end;

function WeidemannHoftrac916DM:writeStream(streamId, connection)
	streamWriteBool(streamId, self.autoSteerBack.isStopped);
end;

function WeidemannHoftrac916DM:mouseEvent(posX, posY, isDown, isUp, button)
end;

function WeidemannHoftrac916DM:keyEvent(unicode, sym, modifier, isDown)
end;

function WeidemannHoftrac916DM:update(dt)

	if self:getIsActiveForInput() then
		if InputBinding.hasEvent(InputBinding.WEIDEMANN_AUTOSTEERBACK) then
			self:setIsAutoSteerBackStopped(not self.autoSteerBack.isStopped);
		end;
	end;
	
end;

function WeidemannHoftrac916DM:updateTick(dt)
	
	if self:getIsActive() and self.isClient then
	
		-- Always update tool attacher to make working easier ;)
		for _, joint in ipairs(self.attacherJoints) do
			if joint.jointIndex ~= 0 then
				setJointFrame(joint.jointIndex, 0, joint.jointTransform);
			end;
		end;
		
		-- Update steering cylinder when steering rotated time changed
		local steeringChanged = false;
		if self.rotatedTime ~= self.rotatedTimeLast then
			self.movingParts[2].isDirty = true;
			self.rotatedTimeLast = self.rotatedTime;
			steeringChanged = true;
		end;		
		
		if self:getIsActiveForInput() then	
		
			-- Move lever 4 speed shift (more than 10 km/h)
			if self.lastSpeed * 3600 >= 10 then
				self.levers[4].moveDir = -1;
			else
				self.levers[4].moveDir = 0;
			end;
			
			-- Check lower motor rpm
			local lowerMotorRpm = false;			
			for i = 1, 3 do
				if self.levers[i].moveDir ~= 0 then
					lowerMotorRpm = true;
					break;
				end;
			end;		
		
			-- Lower motor rpm when steering rotated time changed
			if steeringChanged then
				lowerMotorRpm = true;
			end;
			
			-- Change motor sound pitch offset
			if lowerMotorRpm then
				if self.motorSoundPitchOffset == self.motorSoundPitchOffsetSave then
					self.motorSoundPitchOffset = self.motorSoundPitchOffsetSave - 0.05;
				end;
			else
				if self.motorSoundPitchOffset ~= self.motorSoundPitchOffsetSave then
					self.motorSoundPitchOffset = self.motorSoundPitchOffsetSave;
				end;
			end;			

			-- Tilt cylinder modifikation (because of a not real loader kinematik, normally not needed)
			if self.movingTools[1].moveDiff ~= 0 or self.movingTools[2].moveDiff ~= 0 then
				local rx, ry, rz = getRotation(self.movingTools[1].node);
				local tx, ty, tz = getTranslation(self.movingTools[2].node);
				if rx > math.rad(10) then
					self.movingTools[2].transMin = self.tiltCyl.transMinSave * (1 + (rx - math.rad(10)) / (self.movingTools[1].rotMax - math.rad(10)) * 0.15);
					if tz < self.movingTools[2].transMin then
						self.movingTools[2].curTrans[self.movingTools[2].translationAxis] = self.movingTools[2].transMin;
						setTranslation(self.movingTools[2].node, unpack(self.movingTools[2].curTrans));
					end;
				elseif rx < math.rad(20) then
					self.movingTools[2].transMax = self.tiltCyl.transMaxSave * (1 - math.abs(rx - math.rad(20)) / (math.abs(self.movingTools[1].rotMin) + math.rad(20)) * 0.22);
					if tz > self.movingTools[2].transMax then
						self.movingTools[2].curTrans[self.movingTools[2].translationAxis] = self.movingTools[2].transMax;
						setTranslation(self.movingTools[2].node, unpack(self.movingTools[2].curTrans));
					end;
				end;
			end;
		end;
		
	end;
	
end;

function WeidemannHoftrac916DM:draw()

	-- Auto steer back help text
	g_currentMission:addHelpButtonText(g_i18n:getText("WEIDEMANN_AUTOSTEERBACK"), InputBinding.WEIDEMANN_AUTOSTEERBACK);

end;

function WeidemannHoftrac916DM:attachImplement(implement)
	
	-- Get first moving tool for lever 3
	if implement.object.movingTools ~= nil then
		if implement.object.movingTools[1] ~= nil then
			self.levers[3].movingTool = implement.object.movingTools[1];
		end;
	end;
	
end;

function WeidemannHoftrac916DM:detachImplement(implementIndex, noEventSend)

	-- Remove moving tool for lever 3
	self.levers[3].movingTool = nil;
	self.levers[3].moveDir = 0;
	
end;


function WeidemannHoftrac916DM:setIsAutoSteerBackStopped(isStopped, noEventSend)
    
	if isStopped ~= self.autoSteerBack.isStopped then
		SetIsAutoSteerBackStoppedEvent.sendEvent(self, isStopped, noEventSend);
		self.autoSteerBack.isStopped = isStopped;
		if self.autoSteerBack.isStopped then
			self.maxRotatedTimeSpeed = 0;
		else
			self.maxRotatedTimeSpeed = self.autoSteerBack.maxRotatedTimeSpeedSave;
		end;
	end;
	
end;

function WeidemannHoftrac916DM:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)	
	if not resetVehicles then
		local autoSteerBackStopped = Utils.getNoNil(getXMLBool(xmlFile, key .. "#autoSteerBackStopped"), false);
		self:setIsAutoSteerBackStopped(autoSteerBackStopped);
	end;
    return BaseMission.VEHICLE_LOAD_OK;
end;

function WeidemannHoftrac916DM:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'autoSteerBackStopped="' .. tostring(self.autoSteerBack.isStopped) .. '"';
	return attributes, nil;
end;